#!/usr/bin/python

from kubernetes import client, config
from kubernetes.client.rest import ApiException
from pprint import pprint
import sys

# The aim of this script is to remove failed pods related to jobs that have "COMPLETED" while leaving the "COMPLETED" pods or pods for jobs that have not Completed

# This script searches for all pods in a namespace that aren't running and then checks the status of any associated job and removes failed pods for jobs that suceeded


# Configs can be set in Configuration class directly or using helper utility
config.load_kube_config()
namespace='onap'
if len(sys.argv) == 2:
    namespace = sys.argv[1]
v1 = client.CoreV1Api()
ret = v1.list_pod_for_all_namespaces(watch=False)
api_response = v1.list_namespaced_pod(namespace, pretty=True, field_selector='status.phase!=Running', label_selector='', watch=False)
arrayofinterestingpods=[]
# gather info on all not Running pods i.e. podname and job-name if it exists
for i in api_response.items:
    reason = ""
    if i.status.container_statuses[0].state.terminated != None:
       reason = i.status.container_statuses[0].state.terminated.reason
    if i.status.container_statuses[0].state.running != None:
       reason = i.status.container_statuses[0].state.running.reason
    if i.status.container_statuses[0].state.waiting != None:
       reason = i.status.container_statuses[0].state.waiting.reason
    if 'job-name' in i.metadata.labels:
      tempmap={}
      tempmap['name']= i.metadata.name
      tempmap['status']= reason
      tempmap['job']= i.metadata.labels['job-name']
      arrayofinterestingpods.append(tempmap)


# arrange pods to find completed and not completed pods
resultmap = {}
for item in arrayofinterestingpods:
    if 'job' in item:
       if len(item['job']) != 0:
          if not item['job'] in resultmap:
              temp = {}
              temp['completed'] = []
              temp['notcompleted'] = []
              resultmap[item['job']] = temp

          if "Completed" in item['status']:
             resultmap[item['job']]['completed'].append(item['name'])
          if not "Completed" in item['status']:
             resultmap[item['job']]['notcompleted'].append(item['name'])
podstodelete = []
# decide what has to be deleted
for item in resultmap:
    if len(resultmap[item]['completed']) > 0:
        for needtodelete in resultmap[item]['notcompleted']:
            podstodelete.append(needtodelete)

# delete the unwanted pods
for podtodelete in podstodelete:
   try:
      api_response = v1.delete_namespaced_pod(podtodelete, namespace)
      print("deleted :" +  podtodelete)
   except ApiException as e:
      print("Exception when calling CoreV1Api->delete_namespaced_pod: %s\n" % e)
